// Filename: router.js
define([
    'jquery',
    'underscore',
    'backbone',
    'collections/projects',
    'models/user',
    'views/projects/list',
    'views/projects/view',
    'views/tasks/view',
    'views/users/login',
    'views/users/register'
    //, 'views/users/list'
], function(
    $,
    _,
    Backbone,
    ProjectCollection,
    UserModel,
    ProjectListView,
    ProjectView,
    TaskView,
    LoginView,
    RegisterView
    // , UserListView
){

    $.ajaxSetup({
        statusCode: {
            401: function(){
                // Redirec the to the login page.
                window.location.replace('#/login');
            },
            403: function() {
                // 403 -- Access denied
                window.location.replace('#/denied');
            }
        }
    });

    var AppRouter = Backbone.Router.extend({
        routes: {
            // Define some URL routes
            'projects(/)': 'showProjects',
            'project/:id': 'showProject',
            'task/:project_id/:id': 'showTask',
            '(/)users(/)': 'showUsers',
            '(/)login(/)': 'showLogin',
            '(/)register(/)': 'showRegister',

            // Default
            '*actions': 'showProjects'
        },
        showLogin: function(){
            var loginView = new LoginView({ model: new UserModel({ url: 'api/1/user' })});
            loginView.render();
        },
        showRegister: function(){
            var registerView = new RegisterView({ model: new UserModel({ url: 'api/1/user/register' })});
            registerView.render();
        },
        showProjects: function(){
            var projectListView = new ProjectListView();
            projectListView.render();
        },
        showProject: function(id, showTasks){
            var projectView = new ProjectView({id: id, showTasks: showTasks});
            projectView.render();
        },
        showTask: function(project_id, id){
            var that = this;

            if (!$('#container article.task').length)
            {
                that.showProject(project_id, true);

                $( document ).on( "myCustomEvent", {
                    id: id
                }, function( event, arg1, arg2 ) {
                    that.loadTask(event.data.id);
                });
            } else {
                that.loadTask(id);
            }
        },
        loadTask: function(id){
            var taskView = new TaskView({id: id});
            taskView.render();
        },
        showUsers: function(){
            console.log("showUsers");
        },
        defaultAction: function(){
            console.log("defaultAction");
        }

    });

    var initialize = function(){
        var app_router = new AppRouter;
        Backbone.history.start();
    };
    return {
        initialize: initialize
    };
});
