define([
    'jquery',
    'underscore',
    'backbone',
    'handlebars',
    'etchjs',
    'collections/projects',
    'models/task',
    'assets/templates/project',
    'assets/templates/_article',
    'assets/templates/_task',
    'assets/templates/_new',
    'views/tasks/detail'
], function($, _, Backbone, Handlebars, Etchjs, ProjectsCollection, TaskModel, ProjectTemplate, ArticleTemplate, TaskTemplate, NewTemplate, TaskDetailView) {
    var TaskView = Backbone.View.extend({
        el: $("#container"),

        initialize: function(article) {
            this.id = article.id;
        },

        render: function() {
            var task = new TaskModel(this.id),
                that = this,
                tokenString = localStorage.getItem("comm_token"),
                uid = localStorage.getItem("comm_uid");

            $("body").on("click", ".col-2 .new", function() {
                that.createTask(task.get('id'), 0, 0, 'col-2');
            });

            $("body").on("click", ".col-3 .new", function() {
                that.createTask(task.get('id'), 1, 0, 'col-3');
            });

            task.fetch({data: {id: this.id},
                url: task.url+'/'+this.id,
                headers: { 'Authorization' : uid+':'+tokenString },
                success: function() {
                    console.log($(".task-"+that.id));
                    $(".task-"+that.id).addClass("active").closest("article").addClass("active");
                    $("article").not(".active").remove();
                    $(".task").not(".active").remove();
                    _.each(task.get("comments"), function(comment) {
                        var taskComment = new TaskModel(comment);
                        taskComment.set("type", 0);
                        taskComment.set("change", true);
                        taskComment.set("canChange", true);
                        // taskComment.set("canEdit", true);
                        taskComment.set("canCheck", true);
                        // taskComment.set("canLink", true);
                        taskComment.bind('createTask', that.createTaskFromModel, that);
                        var itemView = new TaskDetailView({model: taskComment});
                        itemView.render();
                        $('article.active .col-2').append(itemView.el);
                    });

                    _.each(task.get("changes"), function(change) {
                        var taskChange = new TaskModel(change);
                        taskChange.set("type", 1);
                        taskChange.set("comment", true);
                        taskChange.set("canComment", true);
                        // taskChange.set("canEdit", true);
                        // taskChange.set("canLink", true);
                        taskChange.set("canCheck", true);
                        taskChange.bind('createTask', that.createTaskFromModel, that);
                        var itemView = new TaskDetailView({model: taskChange});
                        itemView.render();
                        $('article.active .col-3').append(itemView.el);
                    });

                    $(document).on({
                        mouseenter: function () {
                            var $content = $(this).closest(".content");
                            $(".col-3 .content").addClass('medium');
                            if ($(this).hasClass('new')) return;
                            if (parseInt($content.data('id')) !== 0)
                                $(".col-3 .content.comment-"+$content.data("id")).addClass('highlight');
                        },
                        mouseleave: function () {
                            // if ($(this).hasClass('new')) return;
                            $(".col-3 .content").removeClass('medium');
                            $(".col-3 .content").removeClass('highlight');
                        }
                    }, ".col-2 .content");

                    $(document).on({
                        mouseenter: function () {
                            var $content = $(this).closest(".content");
                            $(".col-2 .content").addClass('medium');
                            if ($(this).hasClass('new')) return;
                            if (parseInt($content.data('id')) !== 0)
                                $(".col-2 .content.change-"+$content.data("id")).addClass('highlight');
                        },
                        mouseleave: function () {
                            // if ($(this).hasClass('new')) return;
                            $(".col-2 .content").removeClass('medium');
                            $(".col-2 .content").removeClass('highlight');
                        }
                    }, ".col-3 .content");
            }});

            return this;
        },

        "createTask": function(article, type, link, column) {
            var taskComment = new TaskModel({id: 0, text: ''});
            taskComment.set("type", type);
            taskComment.set("link", link);
            taskComment.set("article", article);
            if (type == 0) {
                taskComment.set("change", true);
                taskComment.set("canChange", true);
            } else {
                taskComment.set("comment", true);
                taskComment.set("canComment", true);
            }
            // taskComment.set("canEdit", true);
            taskComment.set("canCheck", true);
            // taskComment.set("canLink", true);
            taskComment.bind('createTask', this.createTaskFromModel, this);
            var itemView = new TaskDetailView({model: taskComment});
            itemView.render();
            $('article.active .'+column).append(itemView.el);
        },

        "createTaskFromModel" : function(model) {
            if (parseInt(model.get('id')) !== 0)
                this.createTask(model.get('article'), (model.get('type') == 1 ? 0 : 1), model.get('id'),  (model.get('type') == 1 ? 'col-2' : 'col-3'));
        }
    });

    return TaskView;
});
